export declare class AbortError extends Error {
    constructor(message: any);
}
export default function pRetry(input: any, options?: {}): Promise<any>;
export declare function makeRetriable(function_: any, options: any): (...arguments_: any[]) => Promise<any>;
