"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AI_SDK_TOOL_OPERATIONS = exports.AI_SDK_LLM_OPERATIONS = exports.GEN_AI_CHOICE = exports.GEN_AI_ASSISTANT_MESSAGE = exports.GEN_AI_USER_MESSAGE = exports.GEN_AI_SYSTEM_MESSAGE = exports.LANGSMITH_TRACEABLE_PARENT_OTEL_SPAN_ID = exports.LANGSMITH_IS_ROOT = exports.LANGSMITH_TRACEABLE = exports.LANGSMITH_REFERENCE_EXAMPLE_ID = exports.LANGSMITH_USAGE_METADATA = exports.LANGSMITH_PARENT_RUN_ID = exports.LANGSMITH_DOTTED_ORDER = exports.LANGSMITH_TRACE_ID = exports.LANGSMITH_RUN_ID = exports.LANGSMITH_REQUEST_HEADERS = exports.LANGSMITH_REQUEST_STREAMING = exports.LANGSMITH_RUNTIME = exports.LANGSMITH_TAGS = exports.LANGSMITH_METADATA = exports.LANGSMITH_NAME = exports.LANGSMITH_RUN_TYPE = exports.LANGSMITH_SESSION_NAME = exports.LANGSMITH_SESSION_ID = exports.GEN_AI_USAGE_OUTPUT_TOKEN_DETAILS = exports.GEN_AI_USAGE_INPUT_TOKEN_DETAILS = exports.GEN_AI_RESPONSE_SYSTEM_FINGERPRINT = exports.GEN_AI_RESPONSE_SERVICE_TIER = exports.GEN_AI_RESPONSE_ID = exports.GEN_AI_SERIALIZED_DOC = exports.GEN_AI_SERIALIZED_SIGNATURE = exports.GEN_AI_SERIALIZED_NAME = exports.GEN_AI_REQUEST_EXTRA_BODY = exports.GEN_AI_REQUEST_EXTRA_QUERY = exports.GENAI_COMPLETION = exports.GENAI_PROMPT = exports.GEN_AI_RESPONSE_FINISH_REASONS = exports.GEN_AI_REQUEST_PRESENCE_PENALTY = exports.GEN_AI_REQUEST_FREQUENCY_PENALTY = exports.GEN_AI_REQUEST_TOP_P = exports.GEN_AI_REQUEST_TEMPERATURE = exports.GEN_AI_REQUEST_MAX_TOKENS = exports.GEN_AI_USAGE_TOTAL_TOKENS = exports.GEN_AI_USAGE_OUTPUT_TOKENS = exports.GEN_AI_USAGE_INPUT_TOKENS = exports.GEN_AI_RESPONSE_MODEL = exports.GEN_AI_REQUEST_MODEL = exports.GEN_AI_SYSTEM = exports.GEN_AI_OPERATION_NAME = void 0;
// OpenTelemetry GenAI semantic convention attribute names
exports.GEN_AI_OPERATION_NAME = "gen_ai.operation.name";
exports.GEN_AI_SYSTEM = "gen_ai.system";
exports.GEN_AI_REQUEST_MODEL = "gen_ai.request.model";
exports.GEN_AI_RESPONSE_MODEL = "gen_ai.response.model";
exports.GEN_AI_USAGE_INPUT_TOKENS = "gen_ai.usage.input_tokens";
exports.GEN_AI_USAGE_OUTPUT_TOKENS = "gen_ai.usage.output_tokens";
exports.GEN_AI_USAGE_TOTAL_TOKENS = "gen_ai.usage.total_tokens";
exports.GEN_AI_REQUEST_MAX_TOKENS = "gen_ai.request.max_tokens";
exports.GEN_AI_REQUEST_TEMPERATURE = "gen_ai.request.temperature";
exports.GEN_AI_REQUEST_TOP_P = "gen_ai.request.top_p";
exports.GEN_AI_REQUEST_FREQUENCY_PENALTY = "gen_ai.request.frequency_penalty";
exports.GEN_AI_REQUEST_PRESENCE_PENALTY = "gen_ai.request.presence_penalty";
exports.GEN_AI_RESPONSE_FINISH_REASONS = "gen_ai.response.finish_reasons";
exports.GENAI_PROMPT = "gen_ai.prompt";
exports.GENAI_COMPLETION = "gen_ai.completion";
exports.GEN_AI_REQUEST_EXTRA_QUERY = "gen_ai.request.extra_query";
exports.GEN_AI_REQUEST_EXTRA_BODY = "gen_ai.request.extra_body";
exports.GEN_AI_SERIALIZED_NAME = "gen_ai.serialized.name";
exports.GEN_AI_SERIALIZED_SIGNATURE = "gen_ai.serialized.signature";
exports.GEN_AI_SERIALIZED_DOC = "gen_ai.serialized.doc";
exports.GEN_AI_RESPONSE_ID = "gen_ai.response.id";
exports.GEN_AI_RESPONSE_SERVICE_TIER = "gen_ai.response.service_tier";
exports.GEN_AI_RESPONSE_SYSTEM_FINGERPRINT = "gen_ai.response.system_fingerprint";
exports.GEN_AI_USAGE_INPUT_TOKEN_DETAILS = "gen_ai.usage.input_token_details";
exports.GEN_AI_USAGE_OUTPUT_TOKEN_DETAILS = "gen_ai.usage.output_token_details";
// LangSmith custom attributes
exports.LANGSMITH_SESSION_ID = "langsmith.trace.session_id";
exports.LANGSMITH_SESSION_NAME = "langsmith.trace.session_name";
exports.LANGSMITH_RUN_TYPE = "langsmith.span.kind";
exports.LANGSMITH_NAME = "langsmith.trace.name";
exports.LANGSMITH_METADATA = "langsmith.metadata";
exports.LANGSMITH_TAGS = "langsmith.span.tags";
exports.LANGSMITH_RUNTIME = "langsmith.span.runtime";
exports.LANGSMITH_REQUEST_STREAMING = "langsmith.request.streaming";
exports.LANGSMITH_REQUEST_HEADERS = "langsmith.request.headers";
exports.LANGSMITH_RUN_ID = "langsmith.span.id";
exports.LANGSMITH_TRACE_ID = "langsmith.trace.id";
exports.LANGSMITH_DOTTED_ORDER = "langsmith.span.dotted_order";
exports.LANGSMITH_PARENT_RUN_ID = "langsmith.span.parent_id";
exports.LANGSMITH_USAGE_METADATA = "langsmith.usage_metadata";
exports.LANGSMITH_REFERENCE_EXAMPLE_ID = "langsmith.reference_example_id";
exports.LANGSMITH_TRACEABLE = "langsmith.traceable";
exports.LANGSMITH_IS_ROOT = "langsmith.is_root";
exports.LANGSMITH_TRACEABLE_PARENT_OTEL_SPAN_ID = "langsmith.traceable_parent_otel_span_id";
// GenAI event names
exports.GEN_AI_SYSTEM_MESSAGE = "gen_ai.system.message";
exports.GEN_AI_USER_MESSAGE = "gen_ai.user.message";
exports.GEN_AI_ASSISTANT_MESSAGE = "gen_ai.assistant.message";
exports.GEN_AI_CHOICE = "gen_ai.choice";
exports.AI_SDK_LLM_OPERATIONS = [
    "ai.generateText.doGenerate",
    "ai.streamText.doStream",
    "ai.generateObject.doGenerate",
    "ai.streamObject.doStream",
];
exports.AI_SDK_TOOL_OPERATIONS = ["ai.toolCall"];
