import { DefaultReporter } from "@jest/reporters";
declare class LangSmithEvalReporter extends DefaultReporter {
    onTestResult(test: any, testResult: any, aggregatedResults: any): Promise<void>;
}
export default LangSmithEvalReporter;
