/**
 * The base implementation of `assignValue` and `assignMergeValue` without
 * value checks.
 *
 * @private
 * @param {Object} object The object to modify.
 * @param {string} key The key of the property to assign.
 * @param {*} value The value to assign.
 */
declare function baseAssignValue(object: Record<string, any>, key: string, value: any): void;
export default baseAssignValue;
