"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports._REPLICA_TRACE_ROOTS_KEY = exports._LC_CONTEXT_VARIABLES_KEY = void 0;
exports._LC_CONTEXT_VARIABLES_KEY = Symbol.for("lc:context_variables");
exports._REPLICA_TRACE_ROOTS_KEY = Symbol.for("langsmith:replica_trace_roots");
