export declare const DEFAULT_TEST_TIMEOUT = 30000;
export declare const UUID5_NAMESPACE = "6ba7b810-9dad-11d1-80b4-00c04fd430c8";
export declare const STRIP_ANSI_REGEX: RegExp;
export declare const TEST_ID_DELIMITER = ", test_id=";
