export declare function isVersionGreaterOrEqual(current_version: string, target_version: string): boolean;
export declare function parsePromptIdentifier(identifier: string): [string, string, string];
