/**
 * Converts `string` to a property path array.
 *
 * @private
 * @param {string} string The string to convert.
 * @returns {Array} Returns the property path array.
 */
declare const stringToPath: {
    (...args: any[]): any;
    cache: Map<any, any>;
};
export default stringToPath;
