"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TEST_ID_DELIMITER = exports.STRIP_ANSI_REGEX = exports.UUID5_NAMESPACE = exports.DEFAULT_TEST_TIMEOUT = void 0;
exports.DEFAULT_TEST_TIMEOUT = 30_000;
exports.UUID5_NAMESPACE = "6ba7b810-9dad-11d1-80b4-00c04fd430c8";
// From https://stackoverflow.com/a/29497680
exports.STRIP_ANSI_REGEX = 
// eslint-disable-next-line no-control-regex
/[\u001b\u009b][[()#;?]*(?:[0-9]{1,4}(?:;[0-9]{0,4})*)?[0-9A-ORZcf-nqry=><]/g;
exports.TEST_ID_DELIMITER = ", test_id=";
