import type { LanguageModelV2DataContent, LanguageModelV2Message } from "@ai-sdk/provider";
import type { ModelMessage } from "ai";
export declare const normalizeFileDataAsDataURL: (fileData: LanguageModelV2DataContent | ArrayBuffer, mimeType?: string) => string;
export declare const convertMessageToTracedFormat: (message: LanguageModelV2Message | ModelMessage, responseMetadata?: Record<string, unknown>) => Record<string, unknown>;
