"use strict";
/* eslint-disable @typescript-eslint/no-explicit-any */
// Local type definitions to avoid importing @opentelemetry/api
// These match the OpenTelemetry API interfaces but don't require the package to be installed
Object.defineProperty(exports, "__esModule", { value: true });
