/**
 * Get a context variable from a run tree instance
 */
export declare function getContextVar(runTree: any, key: symbol): unknown;
/**
 * Set a context variable on a run tree instance
 */
export declare function setContextVar(runTree: any, key: symbol, value: unknown): void;
