export declare function printReporterTable(testSuiteName: string, results: {
    title: string;
    duration: number;
    status: "pass" | "passed" | "fail" | "failed" | "pending" | "skipped";
}[], testStatus: "pass" | "skip" | "fail" | "passed" | "failed" | "skipped", failureMessage?: string): Promise<void>;
