"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.uuid7FromTime = void 0;
exports.uuid7 = uuid7;
const uuid_1 = require("uuid");
var _uuid_js_1 = require("./utils/_uuid.cjs");
Object.defineProperty(exports, "uuid7FromTime", { enumerable: true, get: function () { return _uuid_js_1.uuid7FromTime; } });
/**
 * Generate a random UUID v7 string.
 */
function uuid7() {
    return (0, uuid_1.v7)();
}
